@@ -12,7 +12,7 @@ use feature ();
 package MooX::Pression;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.100';
+our $VERSION   = '0.200';
 
 use Keyword::Simple ();
 use PPR;
@@ -270,6 +270,7 @@ our $GRAMMAR = qr{
 		
 		(?<MxpAbstractSyntax>
 			
+			(?&PerlOWS)
 			class
 			(?&PerlOWS)
 			(?: [+] )?                                    # CAPTURE:plus
@@ -537,8 +538,13 @@ our $GRAMMAR = qr{
 
 my %_fetch_re_cache;
 sub _fetch_re {
+	my $key = "@_";
 	my $name = shift;
-	$_fetch_re_cache{$name} ||= do {
+	my %opts = @_;
+	
+	$opts{anchor} ||= '';
+	
+	$_fetch_re_cache{$key} ||= do {
 		"$GRAMMAR" =~ m{<$name>(.+)</$name>}s or die "could not fetch re for $name";
 		(my $re = $1) =~ s/\)\#$//;
 		my @lines = split /\n/, $re;
@@ -548,7 +554,9 @@ sub _fetch_re {
 			}
 		}
 		$re = join "\n", @lines;
-		qr/ $re $GRAMMAR /xs;
+		$opts{anchor} eq 'start' ? qr/ ^ $re $GRAMMAR   /xs :
+		$opts{anchor} eq 'end'   ? qr/   $re $GRAMMAR $ /xs :
+		$opts{anchor} eq 'both'  ? qr/ ^ $re $GRAMMAR $ /xs : qr/ $re $GRAMMAR /xs
 	}
 }
 
@@ -597,14 +605,18 @@ sub _handle_signature_list {
 		
 		if ($sig =~ /^\*((?&PerlIdentifier)) $GRAMMAR/xso) {
 			my $name = $1;
-			$parsed[-1]{name} = $name;
+			$parsed[-1]{name}       = $name;
+			$parsed[-1]{named}      = 1;
+			$parsed[-1]{positional} = 0;
 			++$seen_named;
 			$sig =~ s/^\*\Q$name//xs;
 			$sig =~ s/^((?&PerlOWS)) $GRAMMAR//xso;
 		}
 		elsif ($sig =~ /^((?&PerlVariable)) $GRAMMAR/xso) {
 			my $name = $1;
-			$parsed[-1]{name} = $name;
+			$parsed[-1]{name}       = $name;
+			$parsed[-1]{named}      = 0;
+			$parsed[-1]{positional} = 1;
 			++$seen_pos;
 			$sig =~ s/^\Q$name//xs;
 			$sig =~ s/^((?&PerlOWS)) $GRAMMAR//xs;
@@ -630,8 +642,37 @@ sub _handle_signature_list {
 	my @signature_var_list;
 	my $type_params_stuff = '[';
 	
+	my (@head, @tail);
+	if ($seen_named and $seen_pos) {
+		while (@parsed and $parsed[0]{positional}) {
+			push @head, shift @parsed;
+		}
+		while (@parsed and $parsed[-1]{positional}) {
+			unshift @tail, pop @parsed;
+		}
+		if (grep $_->{positional}, @parsed) {
+			require Carp;
+			Carp::croak("Signature contains an unexpected mixture of positional and named parameters");
+		}
+		for my $p (@head, @tail) {
+			my $is_optional = $p->{optional};
+			$is_optional ||= ($p->{type} =~ /^Optional/s);
+			if ($is_optional) {
+				require Carp;
+				Carp::croak("Cannot have optional positional parameter $p->{name} in signature with named parameters");
+			}
+			elsif ($p->{default}) {
+				require Carp;
+				Carp::croak("Cannot have positional parameter $p->{name} with default in signature with named parameters");
+			}
+			elsif ($p->{name} =~ /^[\@\%]/) {
+				require Carp;
+				Carp::croak("Cannot have slurpy parameter $p->{name} in signature with named parameters");
+			}
+		}
+	}
+	
 	require B;
-	die "cannot mix named and positional (yet?)" if $seen_pos && $seen_named;
 
 	my $extra = '';
 	my $count = @parsed;
@@ -672,6 +713,17 @@ sub _handle_signature_list {
 	@signature_var_list = '$arg' if $seen_named;
 	$type_params_stuff .= ']';
 	
+	if (@head or @tail) {
+		require Type::Params;
+		'Type::Params'->VERSION(1.009002);
+		my $head_stuff = join(q[,] => map { $_->{type_is_block} ? sprintf('scalar(do %s)', $_->{type}) : B::perlstring($_->{type}) } @head);
+		my $tail_stuff = join(q[,] => map { $_->{type_is_block} ? sprintf('scalar(do %s)', $_->{type}) : B::perlstring($_->{type}) } @tail);
+		my $opts = sprintf('{head=>%s,tail=>%s},', $head_stuff?"[$head_stuff]":0, $tail_stuff?"[$tail_stuff]":0);
+		substr($type_params_stuff, 1, 0) = $opts; # insert options after "["
+		unshift @signature_var_list, map $_->{name}, @head;
+		push @signature_var_list, map $_->{name}, @tail;
+	}
+	
 	return (
 		$seen_named,
 		join(',', @signature_var_list),
@@ -1106,7 +1158,7 @@ sub import {
 	Keyword::Simple::define class => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpClassSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpClassSyntax', anchor => 'start') or $me->_syntax_error(
 			'class declaration',
 			'class <name> (<signature>) { <block> }',
 			'class <name> { <block> }',
@@ -1128,7 +1180,7 @@ sub import {
 	Keyword::Simple::define abstract => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpAbstractSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpAbstractSyntax', anchor => 'start') or $me->_syntax_error(
 			'abstract class declaration',
 			'abstract class <name> (<signature>) { <block> }',
 			'abstract class <name> { <block> }',
@@ -1151,7 +1203,7 @@ sub import {
 		Keyword::Simple::define $kw => sub {
 			my $ref = shift;
 			
-			$$ref =~ _fetch_re('MxpRoleSyntax') or $me->_syntax_error(
+			$$ref =~ _fetch_re('MxpRoleSyntax', anchor => 'start') or $me->_syntax_error(
 				"$kw declaration",
 				"$kw <name> (<signature>) { <block> }",
 				"$kw <name> { <block> }",
@@ -1173,7 +1225,7 @@ sub import {
 	Keyword::Simple::define toolkit => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpToolkitSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpToolkitSyntax', anchor => 'start') or $me->_syntax_error(
 			'toolkit declaration',
 			'toolkit <toolkit> (<extensions>)',
 			'toolkit <toolkit>;',
@@ -1214,7 +1266,7 @@ sub import {
 		Keyword::Simple::define $kw => sub {
 			my $ref = shift;
 			
-			$$ref =~ _fetch_re('MxpHookSyntax') or $me->_syntax_error(
+			$$ref =~ _fetch_re('MxpHookSyntax', anchor => 'start') or $me->_syntax_error(
 				"$kw hook",
 				"$kw { <block> }",
 				$ref,
@@ -1230,7 +1282,7 @@ sub import {
 	Keyword::Simple::define type_name => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpTypeNameSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpTypeNameSyntax', anchor => 'start') or $me->_syntax_error(
 			'type name declaration',
 			'type_name <identifier>',
 			$ref,
@@ -1245,7 +1297,7 @@ sub import {
 	Keyword::Simple::define extends => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpExtendsSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpExtendsSyntax', anchor => 'start') or $me->_syntax_error(
 			'extends declaration',
 			'extends <classes>',
 			$ref,
@@ -1260,7 +1312,7 @@ sub import {
 	Keyword::Simple::define with => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpWithSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpWithSyntax', anchor => 'start') or $me->_syntax_error(
 			'with declaration',
 			'with <roles>',
 			$ref,
@@ -1276,7 +1328,7 @@ sub import {
 	Keyword::Simple::define requires => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpRequiresSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpRequiresSyntax', anchor => 'start') or $me->_syntax_error(
 			'requires declaration',
 			'requires <name> (<signature>)',
 			'requires <name>',
@@ -1293,7 +1345,7 @@ sub import {
 	Keyword::Simple::define has => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpHasSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpHasSyntax', anchor => 'start') or $me->_syntax_error(
 			'attribute declaration',
 			'has <name> (<spec>) = <default>',
 			'has <name> (<spec>)',
@@ -1315,7 +1367,7 @@ sub import {
 	Keyword::Simple::define constant => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpConstantSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpConstantSyntax', anchor => 'start') or $me->_syntax_error(
 			'constant declaration',
 			'constant <name> = <value>',
 			$ref,
@@ -1332,7 +1384,7 @@ sub import {
 		
 		state $re_attr = _fetch_re('MxpAttribute');
 		
-		$$ref =~ _fetch_re('MxpMethodSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpMethodSyntax', anchor => 'start') or $me->_syntax_error(
 			'method declaration',
 			'method <name> <attributes> (<signature>) { <block> }',
 			'method <name> (<signature>) { <block> }',
@@ -1359,7 +1411,7 @@ sub import {
 		
 		state $re_attr = _fetch_re('MxpAttribute');
 		
-		$$ref =~ _fetch_re('MxpMultiSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpMultiSyntax', anchor => 'start') or $me->_syntax_error(
 			'multimethod declaration',
 			'multi method <name> <attributes> (<signature>) { <block> }',
 			'multi method <name> (<signature>) { <block> }',
@@ -1383,7 +1435,7 @@ sub import {
 			
 			state $re_attr = _fetch_re('MxpAttribute');
 		
-			$$ref =~ _fetch_re('MxpModifierSyntax') or $me->_syntax_error(
+			$$ref =~ _fetch_re('MxpModifierSyntax', anchor => 'start') or $me->_syntax_error(
 				"$kw method modifier declaration",
 				"$kw <name> <attributes> (<signature>) { <block> }",
 				"$kw <name> (<signature>) { <block> }",
@@ -1403,7 +1455,7 @@ sub import {
 	Keyword::Simple::define factory => sub {
 		my $ref = shift;
 		
-		if ( $$ref =~ _fetch_re('MxpFactorySyntax') ) {
+		if ( $$ref =~ _fetch_re('MxpFactorySyntax', anchor => 'start') ) {
 			state $re_attr = _fetch_re('MxpAttribute');
 			my ($pos, $name, $attributes, $sig, $code) = ($+[0], $+{name}, $+{attributes}, $+{sig}, $+{code});
 			my $has_sig = !!exists $+{sig};
@@ -1412,7 +1464,7 @@ sub import {
 			return;
 		}
 		
-		$$ref =~ _fetch_re('MxpFactoryViaSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpFactoryViaSyntax', anchor => 'start') or $me->_syntax_error(
 			'factory method declaration',
 			'factory <name> <attributes> (<signature>) { <block> }',
 			'factory <name> (<signature>) { <block> }',
@@ -1431,7 +1483,7 @@ sub import {
 	Keyword::Simple::define coerce => sub {
 		my $ref = shift;
 		
-		$$ref =~ _fetch_re('MxpCoerceSyntax') or $me->_syntax_error(
+		$$ref =~ _fetch_re('MxpCoerceSyntax', anchor => 'start') or $me->_syntax_error(
 			'coercion declaration',
 			'coerce from <type> via <method_name> { <block> }',
 			'coerce from <type> via <method_name>',
@@ -1584,7 +1636,7 @@ sub _modifier {
 #{
 #	package MooX::Pression::Anonymous::Package;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.100';
+#	our $VERSION   = '0.200';
 #	use overload q[""] => sub { ${$_[0]} }, fallback => 1;
 #	sub DESTROY {}
 #	sub AUTOLOAD {
@@ -1595,7 +1647,7 @@ sub _modifier {
 #	
 #	package MooX::Pression::Anonymous::Class;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.100';
+#	our $VERSION   = '0.200';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	sub new {
 #		my $me = shift;
@@ -1608,12 +1660,12 @@ sub _modifier {
 #	
 #	package MooX::Pression::Anonymous::Role;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.100';
+#	our $VERSION   = '0.200';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	
 #	package MooX::Pression::Anonymous::ParameterizableClass;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.100';
+#	our $VERSION   = '0.200';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -1627,7 +1679,7 @@ sub _modifier {
 #
 #	package MooX::Pression::Anonymous::ParameterizableRole;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.100';
+#	our $VERSION   = '0.200';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -2393,10 +2445,10 @@ bareword name for the method.
   }
 
 MooX::Pression supports method signatures for named arguments and
-positional arguments. If you need a mixture of named and positional
-arguments, this is not currently supported, so instead you should
-define the method with no signature at all, and unpack C<< @_ >> within
-the body of the method.
+positional arguments. A mixture of named and positional arguments
+is allowed, with some limitations. For anything more complicates,
+you should define the method with no signature at all, and unpack
+C<< @_ >> within the body of the method.
 
 =head4 Signatures for Named Arguments
 
@@ -2519,6 +2571,54 @@ pretend it is a hashref or arrayref.
     ...;
   }
 
+=head4 Signatures with Mixed Arguments
+
+Since MooX::Pression 0.200, you may mix named and positional arguments
+with the following limitations:
+
+=over
+
+=item *
+
+Positional arguments must appear at the beginning and/or end of the list.
+They cannot be surrounded by named arguments.
+
+=item *
+
+Positional arguments cannot be optional and cannot have a default. They
+must be required. (Named arguments can be optional and have defaults.)
+
+=item *
+
+No slurpies!
+
+=back
+
+  method print_html ($tag, Str $text, *htmlver?, *xml?, $fh) {
+  
+    confess "update your HTML" if $arg->htmlver < 5;
+    
+    if (length $text) {
+      print $fh "<tag>$text</tag>";
+    }
+    elsif ($arg->xml) {
+      print $fh "<tag />";
+    }
+    else {
+      print $fh "<tag></tag>";
+    }
+  }
+  
+  $obj->print_html('h1', 'Hello World', { xml => true }, \*STDOUT);
+  $obj->print_html('h1', 'Hello World',   xml => true  , \*STDOUT);
+  $obj->print_html('h1', 'Hello World',                  \*STDOUT);
+
+Mixed signatures are basically implemented like named signatures, but
+prior to interpreting C<< @_ >> as a hash, some parameters are spliced
+off the head and tail. We need to know how many elements to splice off
+each end, so that is why there are restrictions on slurpies and optional
+parameters.
+
 =head4 Empty Signatures
 
 There is a difference between the following two methods:
